@@ -5,7 +5,6 @@ from xml.sax.saxutils import escape
 from hashedindex import HashedIndex
 from hashedindex.textparser import word_tokenize
 
-from hashedixsearch.analysis import SynonymAnalyzer
 from hashedixsearch._utils import (
     _is_separator,
     _render_match,
@@ -18,32 +17,25 @@ class HashedIXSearch(object):
         ngrams=4,
         stemmer=None,
         stopwords=None,
-        synonyms=None,
-        retain_casing=False,
-        retain_punctuation=False,
-        retain_whitespace=False,
+        casing=False,
+        punctuation=False,
+        whitespace=False,
     ):
         self.index = HashedIndex()
         self.ngrams = ngrams
         self.stemmer = stemmer
         self.stopwords = stopwords
-        self.synonyms = synonyms
-        self.retain_casing = retain_casing
-        self.retain_punctuation = retain_punctuation
-        self.retain_whitespace = retain_whitespace
+        self.casing = casing
+        self.punctuation = punctuation
+        self.whitespace = whitespace
 
     def tokenize(self, doc, **kwargs):
         ngrams = kwargs.get("ngrams", self.ngrams)
         stemmer = kwargs.get("stemmer", self.stemmer)
         stopwords = kwargs.get("stopwords", self.stopwords)
-        synonyms = kwargs.get("synonyms", self.synonyms)
-        retain_casing = kwargs.get("retain_casing", self.retain_casing)
-        retain_punctuation = kwargs.get("retain_punctuation", self.retain_punctuation)
-        retain_whitespace = kwargs.get("retain_whitespace", self.retain_whitespace)
-
-        if synonyms:
-            analyzer = SynonymAnalyzer(synonyms)
-            doc = str().join(analyzer.process(doc))
+        casing = kwargs.get("casing", self.casing)
+        punctuation = kwargs.get("punctuation", self.punctuation)
+        whitespace = kwargs.get("whitespace", self.whitespace)
 
         for ngrams in range(ngrams, 0, -1):
             yield from word_tokenize(
@@ -52,9 +44,9 @@ class HashedIXSearch(object):
                 stemmer=stemmer,
                 stopwords=stopwords or [],
                 ignore_numeric=False,
-                retain_casing=retain_casing,
-                retain_punctuation=retain_punctuation,
-                tokenize_whitespace=retain_whitespace,
+                retain_casing=casing,
+                retain_punctuation=punctuation,
+                tokenize_whitespace=whitespace,
             )
 
         # Produce an end-of-stream marker
@@ -114,27 +106,35 @@ class HashedIXSearch(object):
             doc=doc,
             ngrams=1,
             stemmer=None,
-            retain_casing=True,
-            retain_punctuation=True,
-            retain_whitespace=True,
+            casing=True,
+            punctuation=True,
+            whitespace=True,
         )
         while unstemmed_token := next(unstemmed_tokens):
             stemmed_token = next(
                 self.tokenize(
                     doc=unstemmed_token[0],
                     ngrams=1,
-                    retain_casing=case_sensitive,
-                    retain_punctuation=True,
-                    retain_whitespace=True,
+                    casing=case_sensitive,
+                    punctuation=True,
+                    whitespace=True,
                 )
             )
             yield unstemmed_token[0], stemmed_token[0]
 
-    def highlight(self, doc, terms, case_sensitive=True, term_attributes=None):
+    def highlight(
+        self,
+        doc,
+        terms,
+        case_sensitive=True,
+        term_attributes=None,
+        limit=None,
+    ):
         # If no terms are provided to match on, do not attempt highlighting
         if not terms:
             return escape(doc)
 
+        limit = limit or -1
         term_attributes = term_attributes or {}
         token_pairs = self._token_pairs(doc, case_sensitive)
 
@@ -149,14 +149,19 @@ class HashedIXSearch(object):
         for unstemmed_token, stemmed_token in token_pairs:
 
             # Advance the match window for each candidate term
-            if not _is_separator(stemmed_token):
-                candidates = candidates or {term: term for term in terms}
+            if not _is_separator(stemmed_token) and not limit == 0:
+                candidates = candidates or {term: term for term in terms if term}
                 candidates = {
                     term: tokens[1:]
                     for term, tokens in candidates.items()
                     if tokens[0] == stemmed_token
                 }
 
+            # We had a partial match, but it was lost
+            if accumulator.tell() and not candidates:
+                markup.write(accumulator.getvalue())
+                accumulator = StringIO()
+
             # Prepare the current token for output, and write it to the
             # accumulator buffer if we are within candidate highlight tokens
             output = escape(unstemmed_token)
@@ -168,6 +173,7 @@ class HashedIXSearch(object):
             if emit:
                 attributes = term_attributes.get(emit)
                 output = _render_match(accumulator, attributes)
+                limit -= 1
                 candidates = {}
                 accumulator = StringIO()
 
